@@ -87,7 +87,11 @@ struct pridef {
 	char    *cntcat;        /* used categories of counters            */
 	char    flag;           /* flag on command line                   */
 	void    (*prihead)();   /* print header of list                   */
-	int     (*priline)();   /* print counters per line (excl. time)   */
+	int     (*priline)(struct sstat *, struct tstat *, struct tstat **,
+		           int, time_t, time_t, time_t,
+		           int, int, int, char *,
+        	           int, int, int, int, int, int);
+		                /* print counters per line (excl. time)   */
 	char    *about;         /* statistics about what                  */
 };
 
@@ -233,6 +237,8 @@ atopsar(int argc, char *argv[])
 			}
 		}
 
+		free(flaglist);
+
 		/*
 		** get optional interval-value and
 		** optional number of samples	
@@ -471,7 +477,8 @@ engine(void)
 
 		photosyst(cursstat);	/* obtain new counters      */
 
-		deviatsyst(cursstat, presstat, devsstat);
+		deviatsyst(cursstat, presstat, devsstat,
+				 curtime-pretime > 0 ? curtime-pretime : 1);
 
 		/*
 		** activate the report-function to visualize the deviations
@@ -515,10 +522,12 @@ reportlive(time_t curtime, int numsecs, struct sstat *ss)
 			/*
 			** print header-line
 			*/
+			printf("\n");
+
 			if (usecolors)
 				printf(COLSETHEAD);
 
-			printf("\n%s  ", convtime(curtime-numsecs, timebuf));
+			printf("%s  ", convtime(curtime-numsecs, timebuf));
 	
 			(pridef[i].prihead)(osvers, osrel, ossub);
 	
@@ -532,7 +541,7 @@ reportlive(time_t curtime, int numsecs, struct sstat *ss)
 			*/
 			printf("%s  ", convtime(curtime, timebuf));
 	
-			if ( !(pridef[i].priline)(ss, (struct tstat *)0, 0,
+			if ( !(pridef[i].priline)(ss, (struct tstat *)0, 0, 0,
 				numsecs, numsecs*hertz, hertz,
 				osvers, osrel, ossub,
 				stampalways ? timebuf : "        ",
@@ -578,10 +587,12 @@ reportlive(time_t curtime, int numsecs, struct sstat *ss)
 			/*
 			** print header-line
 			*/
+			printf("\n");
+
 			if (usecolors)
 				printf(COLSETHEAD);
 
-			printf("\n%s  ", convtime(curtime, timebuf));
+			printf("%s  ", convtime(curtime, timebuf));
 	
 			(pridef[i].prihead)(osvers, osrel, ossub);
 
@@ -601,7 +612,7 @@ reportlive(time_t curtime, int numsecs, struct sstat *ss)
 		*/
 		printf("%s  ", convtime(curtime, timebuf));
 	
-		if ( !(rv = (pridef[i].priline)(ss, (struct tstat *)0, 0,
+		if ( !(rv = (pridef[i].priline)(ss, (struct tstat *)0, 0, 0,
 					numsecs, numsecs*hertz, hertz,
 					osvers, osrel, ossub, 
 		                        stampalways ? timebuf : "        ",
@@ -623,10 +634,12 @@ reportlive(time_t curtime, int numsecs, struct sstat *ss)
 			/*
 			** print header-line
 			*/
+			printf("\n");
+
 			if (usecolors)
 				printf(COLSETHEAD);
 
-			printf("\n%s  ", convtime(curtime, timebuf));
+			printf("%s  ", convtime(curtime, timebuf));
 	
 			(pridef[i].prihead)(osvers, osrel, ossub);
 
@@ -717,10 +730,12 @@ reportraw(time_t curtime, int numsecs,
 		/*
 		** print header-line
 		*/
+		printf("\n");
+
 		if (usecolors)
 			printf(COLSETHEAD);
 
-		printf("\n%s  ", convtime(pretime, timebuf));
+		printf("%s  ", convtime(pretime, timebuf));
 
 		(pridef[prinow].prihead)(osvers, osrel, ossub);
 
@@ -747,7 +762,7 @@ reportraw(time_t curtime, int numsecs,
 			printf("%s  ", convtime(lasttime, timebuf));
 
 			rv = (pridef[prinow].priline)(&totsyst,
-				(struct tstat *)0, 0,
+				(struct tstat *)0, 0, 0,
 				totalsec, totalsec*hertz, hertz,
 			        osvers, osrel, ossub,
 		                stampalways ? timebuf : "        ",
@@ -856,7 +871,7 @@ reportraw(time_t curtime, int numsecs,
 			printf("%s  ", convtime(curtime, timebuf));
 
 			rv = (pridef[prinow].priline) (&totsyst,
-					(struct tstat *)0, 0,
+					(struct tstat *)0, 0, 0,
 					totalsec, totalsec*hertz, hertz,
 					osvers, osrel, ossub,
 					stampalways ? timebuf : "        ",
@@ -977,6 +992,12 @@ pratopsaruse(char *myname)
 		fprintf(stderr,
 		"\t  -%c  %s\n", pridef[i].flag, pridef[i].about);
 
+	fprintf(stderr, "\n");
+	fprintf(stderr,
+                "Please refer to the man-page of 'atopsar' "
+	        "for more details.\n");
+
+
 	cleanstop(1);
 }
 
@@ -1149,8 +1170,7 @@ cpuline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
         cputot = ss->cpu.all.stime + ss->cpu.all.utime +
                  ss->cpu.all.ntime + ss->cpu.all.itime +
                  ss->cpu.all.wtime + ss->cpu.all.Itime +
-                 ss->cpu.all.Stime + ss->cpu.all.steal +
-                 ss->cpu.all.guest;
+                 ss->cpu.all.Stime + ss->cpu.all.steal;
 
 	if (cputot == 0)
 		cputot = 1;	/* avoid divide-by-zero */
@@ -1186,8 +1206,7 @@ cpuline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
         		cputot = ss->cpu.cpu[i].stime + ss->cpu.cpu[i].utime +
                  	         ss->cpu.cpu[i].ntime + ss->cpu.cpu[i].itime +
                  	         ss->cpu.cpu[i].wtime + ss->cpu.cpu[i].Itime +
-                 	         ss->cpu.cpu[i].Stime + ss->cpu.cpu[i].steal +
-                 	         ss->cpu.cpu[i].guest;
+                 	         ss->cpu.cpu[i].Stime + ss->cpu.cpu[i].steal;
 
 			if (cputot == 0)
 				cputot = 1;	/* avoid divide-by-zero */
@@ -1276,14 +1295,14 @@ taskline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 		printf("%8.2lf %7.2lf  %7d %7d    %6d %7d %7d\n",
 			(double)ss->cpu.nprocs / deltasec,
 			(double)pexit          / deltasec,
-			ppres, pzombie, ntrun, ntslpi, ntslpu);
+			nactproc-pexit, pzombie, ntrun, ntslpi, ntslpu);
 	}
 	else
 	{
 		printf("%8.2lf %7.2lf  %7d %7d\n",
 			(double)ss->cpu.nprocs / deltasec,
 			(double)pexit          / deltasec,
-			ppres, pzombie);
+			nactproc-pexit, pzombie);
 	}
 
 	return 1;
@@ -1445,13 +1464,11 @@ gendskline(struct sstat *ss, char *tstamp, char selector)
         mstot  = (ss->cpu.all.stime + ss->cpu.all.utime +
                   ss->cpu.all.ntime + ss->cpu.all.itime +
                   ss->cpu.all.wtime + ss->cpu.all.Itime +
-                  ss->cpu.all.Stime + ss->cpu.all.steal +
-                  ss->cpu.all.guest                      )
+                  ss->cpu.all.Stime + ss->cpu.all.steal  )
 				* (count_t)1000 / hertz / ss->cpu.nrcpu;
 
 	for (i=0; i < nunit; i++, dp++)
 	{
-		char	buf[32];
 		char	*pn;
 		int	len;
 
@@ -1477,7 +1494,6 @@ gendskline(struct sstat *ss, char *tstamp, char selector)
 		else
 			pn = dp->name;
 
-        sprintf(buf, "%12.12s", pn);
 		printf("%-14s %3.0lf%% %6.1lf %7.1lf %7.1lf %7.1lf "
 		       "%5.1lf %6.2lf ms",
 		    	pn,
@@ -1532,6 +1548,121 @@ dskline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 	return gendskline(ss, tstamp, 'd');
 }
 
+/*
+** NFS client statistics
+*/
+static void
+nfmhead(int osvers, int osrel, int ossub)
+{
+	printf("mounted_device                          physread/s  physwrit/s"
+               "  _nfm_");
+}
+
+static int
+nfmline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
+        time_t deltasec, time_t deltatic, time_t hz,
+        int osvers, int osrel, int ossub, char *tstamp,
+        int ppres,  int ntrun, int ntslpi, int ntslpu, int pexit, int pzombie)
+{
+	static char	firstcall = 1;
+	register long	i, nlines = 0;
+	char		*pn, state;
+	int		len;
+
+	for (i=0; i < ss->nfs.nfsmounts.nrmounts; i++)	/* per NFS mount */
+	{
+		/*
+		** print for the first sample all mounts that
+		** are found; afterwards print only the mounts
+		** that were really active during the interval
+		*/
+		if (firstcall                                  ||
+		    allresources                               ||
+		    ss->nfs.nfsmounts.nfsmnt[i].age < deltasec ||
+		    ss->nfs.nfsmounts.nfsmnt[i].bytestotread   ||
+		    ss->nfs.nfsmounts.nfsmnt[i].bytestotwrite    )
+		{
+			if (nlines++)
+				printf("%s  ", tstamp);
+
+			if ( (len = strlen(ss->nfs.nfsmounts.nfsmnt[i].mountdev)) > 38)
+				pn = ss->nfs.nfsmounts.nfsmnt[i].mountdev + len - 38;
+			else
+				pn = ss->nfs.nfsmounts.nfsmnt[i].mountdev;
+
+		    	if (ss->nfs.nfsmounts.nfsmnt[i].age < deltasec)
+				state = 'M';
+			else
+				state = ' ';
+
+			printf("%-38s %10.3lfK %10.3lfK    %c\n", 
+			    pn,
+			    (double)ss->nfs.nfsmounts.nfsmnt[i].bytestotread  /
+								1024 / deltasec,
+			    (double)ss->nfs.nfsmounts.nfsmnt[i].bytestotwrite /
+								1024 / deltasec,
+			    state);
+		}
+	}
+
+	if (nlines == 0)
+	{
+		printf("\n");
+		nlines++;
+	}
+
+	firstcall= 0;
+	return nlines;
+}
+
+static void
+nfchead(int osvers, int osrel, int ossub)
+{
+	printf("     rpc/s   rpcread/s  rpcwrite/s  retrans/s  autrefresh/s   "
+               "  _nfc_");
+}
+
+static int
+nfcline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
+        time_t deltasec, time_t deltatic, time_t hz,
+        int osvers, int osrel, int ossub, char *tstamp,
+        int ppres,  int ntrun, int ntslpi, int ntslpu, int pexit, int pzombie)
+{
+	printf("%10.2lf  %10.2lf  %10.2lf %10.2lf  %12.2lf\n",
+		(double)ss->nfs.client.rpccnt        / deltasec,
+		(double)ss->nfs.client.rpcread       / deltasec,
+		(double)ss->nfs.client.rpcwrite      / deltasec,
+		(double)ss->nfs.client.rpcretrans    / deltasec,
+		(double)ss->nfs.client.rpcautrefresh / deltasec);
+
+	return 1;
+}
+
+static void
+nfshead(int osvers, int osrel, int ossub)
+{
+	printf("  rpc/s  rpcread/s rpcwrite/s MBcr/s  MBcw/s  "
+               "nettcp/s netudp/s _nfs_");
+}
+
+static int
+nfsline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
+        time_t deltasec, time_t deltatic, time_t hz,
+        int osvers, int osrel, int ossub, char *tstamp,
+        int ppres,  int ntrun, int ntslpi, int ntslpu, int pexit, int pzombie)
+{
+	printf("%7.2lf %10.2lf %10.2lf %6.2lf %7.2lf %9.2lf %8.2lf\n",
+		(double)ss->nfs.server.rpccnt    / deltasec,
+		(double)ss->nfs.server.rpcread   / deltasec,
+		(double)ss->nfs.server.rpcwrite  / deltasec,
+		(double)ss->nfs.server.nrbytes / 1024.0 / 1024.0 / deltasec,
+		(double)ss->nfs.server.nwbytes / 1024.0 / 1024.0 / deltasec,
+		(double)ss->nfs.server.nettcpcnt / deltasec,
+		(double)ss->nfs.server.netudpcnt / deltasec);
+
+	return 1;
+}
+
 /*
 ** network-interface statistics
 */
@@ -1553,6 +1684,8 @@ ifline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 	double		busy;
 	char		busyval[16], dupval;
 	unsigned int	badness;
+	char		*pn;
+	int		len;
 
 	for (i=0; i < ss->intf.nrintf; i++)	/* per interface */
 	{
@@ -1587,6 +1720,25 @@ ifline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 				busy = (ival + oval) * 100 /
 				        ss->intf.intf[i].speed;
 
+			// especially with wireless, the speed might have
+			// dropped temporarily to a very low value (snapshot)
+			// it might be better to take the speed of the
+			// previous sample
+			if (busy > 100 && ss->intf.intf[i].speed <
+			                  	ss->intf.intf[i].speedp )
+			{
+				ss->intf.intf[i].speed =
+					ss->intf.intf[i].speedp;
+
+				if (ss->intf.intf[i].duplex)
+					busy = (ival > oval ?
+						ival*100 : oval*100) /
+				        	ss->intf.intf[i].speed;
+				else
+					busy = (ival + oval) * 100 /
+				        	ss->intf.intf[i].speed;
+			}
+
 			snprintf(busyval, sizeof busyval,
 						"%3.0lf%%", busy);
 		}
@@ -1616,11 +1768,16 @@ ifline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 		else
 			badness = 0;
 
+		if ( (len = strlen(ss->intf.intf[i].name)) > 6)
+			pn = ss->intf.intf[i].name + len - 6;
+		else
+			pn = ss->intf.intf[i].name;
+
 		preprint(badness);
 
 		printf("%-6s %4s %7.1lf %7.1lf %8.0lf %8.0lf "
 		       "%5lld %5lld %7ld %c", 
-			ss->intf.intf[i].name, busyval,
+			pn, busyval,
 			(double)ss->intf.intf[i].rpack / deltasec,
 			(double)ss->intf.intf[i].spack / deltasec,
 			(double)ss->intf.intf[i].rbyte / 1024 / deltasec,
@@ -1656,6 +1813,8 @@ IFline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 {
 	static char	firstcall = 1;
 	register long	i, nlines = 0;
+	char		*pn;
+	int		len;
 
 	for (i=0; i < ss->intf.nrintf; i++)	/* per interface */
 	{
@@ -1671,9 +1830,14 @@ IFline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 		if (nlines++)
 			printf("%s  ", tstamp);
 
+		if ( (len = strlen(ss->intf.intf[i].name)) > 6)
+			pn = ss->intf.intf[i].name + len - 6;
+		else
+			pn = ss->intf.intf[i].name;
+
 		printf("%-6s %6.2lf %6.2lf %6.2lf %7.2lf %7.2lf "
 		       "%8.2lf %10.2lf\n", 
-			ss->intf.intf[i].name,
+			pn,
 			(double)ss->intf.intf[i].rerrs    / deltasec,
 			(double)ss->intf.intf[i].serrs    / deltasec,
 			(double)ss->intf.intf[i].scollis  / deltasec,
@@ -2060,8 +2224,7 @@ topcline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 	availcpu  = ss->cpu.all.stime + ss->cpu.all.utime +
 	            ss->cpu.all.ntime + ss->cpu.all.itime +
 	            ss->cpu.all.wtime + ss->cpu.all.Itime +
-		    ss->cpu.all.Stime + ss->cpu.all.steal +
-		    ss->cpu.all.guest; 
+		    ss->cpu.all.Stime + ss->cpu.all.steal;
 
 	availcpu /= ss->cpu.nrcpu;
 
@@ -2293,6 +2456,9 @@ struct pridef pridef[] =
    {0,  "cd", 'l',  lvmhead,	lvmline,	"logical volume activity", },
    {0,  "cd", 'f',  mddhead,	mddline,	"multiple device activity",},
    {0,  "cd", 'd',  dskhead,	dskline,	"disk activity",          },
+   {0,  "n",  'n',  nfmhead,	nfmline,	"NFS client mounts",      },
+   {0,  "n",  'j',  nfchead,	nfcline,	"NFS client activity",    },
+   {0,  "n",  'J',  nfshead,	nfsline,	"NFS server activity",    },
    {0,  "n",  'i',  ifhead,	ifline,		"net-interf (general)",   },
    {0,  "n",  'I',  IFhead,	IFline,		"net-interf (errors)",    },
    {0,  "n",  'w',  ipv4head,	ipv4line,	"ip   v4    (general)",   },
