@@ -197,10 +197,11 @@ static void calcdiff(struct tstat *, struct tstat *, struct tstat *,
 ** calculate the process-activity during the last sample
 */
 int
-deviatproc(struct tstat *aproc, int npresent,
-           struct tstat *eproc, int nexit, int deviatonly,
-	   struct tstat *dproc, struct sstat *dstat, unsigned int *nactproc,
-           int *totproc, int *totrun, int *totslpi, int *totslpu, int *nzombie)
+deviattask(struct tstat *curtpres, int ntaskpres,
+           struct tstat *curpexit, int nprocexit, int deviatonly,
+	   struct tstat *devtstat, struct sstat *devsstat,
+           unsigned int *nprocdev, int *nprocpres,
+           int *totrun, int *totslpi, int *totslpu, int *totzombie)
 {
 	register int		c, d;
 	register struct tstat	*curstat, *devstat, *procstat = 0;
@@ -212,11 +213,10 @@ deviatproc(struct tstat *aproc, int npresent,
 	/*
 	** needed for sanity check later on...
 	*/
-	totusedcpu	= dstat->cpu.all.stime + dstat->cpu.all.utime +
-			  dstat->cpu.all.ntime + dstat->cpu.all.itime +
-			  dstat->cpu.all.wtime + dstat->cpu.all.Itime +
-			  dstat->cpu.all.Stime + dstat->cpu.all.steal +
-			  dstat->cpu.all.guest;
+	totusedcpu	= devsstat->cpu.all.stime + devsstat->cpu.all.utime +
+			  devsstat->cpu.all.ntime + devsstat->cpu.all.itime +
+			  devsstat->cpu.all.wtime + devsstat->cpu.all.Itime +
+			  devsstat->cpu.all.Stime + devsstat->cpu.all.steal;
 
 	/*
 	** make new list of all tasks in the process-database;
@@ -228,21 +228,21 @@ deviatproc(struct tstat *aproc, int npresent,
 	/*
 	** calculate deviations per present process
 	*/
-	*totproc=*totrun=*totslpi=*totslpu=*nzombie= 0;
+	*nprocpres = *totrun = *totslpi = *totslpu = *totzombie = 0;
 
-	for (c=0, d=0, *nactproc=0; c < npresent; c++)
+	for (c=0, d=0, *nprocdev=0; c < ntaskpres; c++)
 	{
 		char	newtask = 0;
 
-		curstat = aproc+c;
+		curstat = curtpres+c;
 
 		if (curstat->gen.isproc)
 		{
-			(*totproc)++;
+			(*nprocpres)++;
 
 			if (curstat->gen.state == 'Z')
 			{
-				(*nzombie)++;
+				(*totzombie)++;
 			}
 			else
 			{
@@ -321,7 +321,7 @@ deviatproc(struct tstat *aproc, int npresent,
 		if (curstat->gen.isproc)
 		{
 			procsaved = 1;
-			(*nactproc)++;
+			(*nprocdev)++;
 		}
 		else
 		{
@@ -330,17 +330,16 @@ deviatproc(struct tstat *aproc, int npresent,
 			*/
 			if (!procsaved)
 			{
-				devstat = dproc+d;
+				devstat = devtstat+d;
 				calcdiff(devstat, procstat, procstat, 0,
 								totusedcpu);
 				procsaved = 1;
-				(*nactproc)++;
+				(*nprocdev)++;
 				d++;
 			}
 		}
 
-		devstat = dproc+d;
-
+		devstat = devtstat+d;
 
 		calcdiff(devstat, curstat, &prestat, newtask, totusedcpu);
 		d++;
@@ -349,9 +348,9 @@ deviatproc(struct tstat *aproc, int npresent,
 	/*
 	** calculate deviations per exited process
 	*/
-	if (nexit > 0 && supportflags&NETATOPD)
+	if (nprocexit > 0 && supportflags&NETATOPD)
 	{
-		if (eproc->gen.pid)
+		if (curpexit->gen.pid)
 			hashtype = 'p';
 		else
 			hashtype = 'b';
@@ -359,7 +358,7 @@ deviatproc(struct tstat *aproc, int npresent,
 		netatop_exithash(hashtype);
 	}
 
-	for (c=0; c < nexit; c++)
+	for (c=0; c < nprocexit; c++)
 	{
 		/*
 		** check if this process has been started AND
@@ -367,7 +366,7 @@ deviatproc(struct tstat *aproc, int npresent,
 		** if so, it has no use to check if there is still 
 		** existing info present in the process-database
 		*/
-		curstat = eproc+c;
+		curstat = curpexit+c;
 
 		if (curstat->gen.pid)	/* acctrecord contains pid? */
 		{
@@ -403,7 +402,7 @@ deviatproc(struct tstat *aproc, int npresent,
 		/*
 		** now do the calculations
 		*/
-		devstat = dproc+d;
+		devstat = devtstat+d;
 		memset(devstat, 0, sizeof *devstat);
 
 		devstat->gen        = curstat->gen;
@@ -457,7 +456,7 @@ deviatproc(struct tstat *aproc, int npresent,
 		}
 
 		d++;
-		(*nactproc)++;
+		(*nprocdev)++;
 
 		if (prestat.gen.pid > 0)
 			pdb_deltask(prestat.gen.pid, prestat.gen.isproc);
@@ -525,6 +524,7 @@ calcdiff(struct tstat *devstat, struct tstat *curstat, struct tstat *prestat,
 	devstat->mem.vexec  = curstat->mem.vexec;
 	devstat->mem.vmem   = curstat->mem.vmem;
 	devstat->mem.rmem   = curstat->mem.rmem;
+	devstat->mem.pmem   = curstat->mem.pmem;
 	devstat->mem.vgrow  = curstat->mem.vmem   - prestat->mem.vmem;
 	devstat->mem.rgrow  = curstat->mem.rmem   - prestat->mem.rmem;
 	devstat->mem.vdata  = curstat->mem.vdata;
@@ -594,10 +594,12 @@ calcdiff(struct tstat *devstat, struct tstat *curstat, struct tstat *prestat,
 ** calculate the system-activity during the last sample
 */
 void
-deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev)
+deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev,
+							long interval)
 {
 	register int	i, j;
 	count_t		*cdev, *ccur, *cpre;
+	struct ifprop	ifprop;
 
 	dev->cpu.nrcpu     = cur->cpu.nrcpu;
 	dev->cpu.devint    = subcount(cur->cpu.devint, pre->cpu.devint);
@@ -675,6 +677,12 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev)
 	dev->mem.shmrss		= cur->mem.shmrss;
 	dev->mem.shmswp		= cur->mem.shmswp;
 
+	dev->mem.tothugepage	= cur->mem.tothugepage;
+	dev->mem.freehugepage	= cur->mem.freehugepage;
+	dev->mem.hugepagesz	= cur->mem.hugepagesz;
+
+	dev->mem.vmwballoon	= cur->mem.vmwballoon;
+
 	dev->mem.swouts		= subcount(cur->mem.swouts,  pre->mem.swouts);
 	dev->mem.swins		= subcount(cur->mem.swins,   pre->mem.swins);
 	dev->mem.pgscans	= subcount(cur->mem.pgscans, pre->mem.pgscans);
@@ -766,57 +774,68 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev)
 
 	/*
 	** calculate deviations for interfaces
+	**
+	** refresh all interface properties
 	*/
+        regainrootprivs();	/* get root privileges      */
+
+	initifprop();		/* refresh interface info   */
+
+	if (! droprootprivs())  /* drop setuid-root privs   */
+		cleanstop(42);
+
+	for (i=0; cur->intf.intf[i].name[0]; i++)
+	{
+		strcpy(ifprop.name, cur->intf.intf[i].name);
+
+		getifprop(&ifprop);
+
+		cur->intf.intf[i].type   = ifprop.type;
+		cur->intf.intf[i].speed  = ifprop.speed;
+		cur->intf.intf[i].speedp = ifprop.speed;
+		cur->intf.intf[i].duplex = ifprop.fullduplex;
+	}
+
 	if (pre->intf.intf[0].name[0] == '\0')	/* first sample? */
 	{
-		struct ifprop	ifprop;
-
 		for (i=0; cur->intf.intf[i].name[0]; i++)
 		{
 			strcpy(pre->intf.intf[i].name, cur->intf.intf[i].name);
 
-			strcpy(ifprop.name, cur->intf.intf[i].name);
-
-			getifprop(&ifprop);
-
-			pre->intf.intf[i].speed         = ifprop.speed;
-			pre->intf.intf[i].duplex        = ifprop.fullduplex;
+			pre->intf.intf[i].type   = cur->intf.intf[i].type;
+			pre->intf.intf[i].speed  = cur->intf.intf[i].speed;
+			pre->intf.intf[i].speedp = cur->intf.intf[i].speedp;
+			pre->intf.intf[i].duplex = cur->intf.intf[i].duplex;
  		}
 	}
 	
-	for (i=0; cur->intf.intf[i].name[0]; i++)
+	for (i=0, j=0; cur->intf.intf[i].name[0]; i++, j++)
 	{
 		/*
-		** check if an interface has been added or removed;
-		** in that case, skip further handling for this sample
+		** be sure that we have the same interface
+		** (interfaces could have been added or removed since
+		** previous sample)
 		*/
-		if (strcmp(cur->intf.intf[i].name, pre->intf.intf[i].name) != 0)
+		if (strcmp(cur->intf.intf[i].name, pre->intf.intf[j].name) != 0)
 		{
-			int		j;
-			struct ifprop	ifprop;
-
-			/*
-			** take care that interface properties are
-			** corrected for future samples
-			*/
-                        regainrootprivs();	/* get root privileges      */
-
-		        initifprop();		/* refresh interface info   */
-
-			if (! droprootprivs())  /* drop setuid-root privs   */
-				cleanstop(42);
-
-			for (j=0; cur->intf.intf[j].name[0]; j++)
+			// try to resync
+			for (j=0; pre->intf.intf[j].name[0]; j++)
 			{
-				strcpy(ifprop.name, cur->intf.intf[j].name);
-
-				getifprop(&ifprop);
-
-				cur->intf.intf[j].speed  = ifprop.speed;
-				cur->intf.intf[j].duplex = ifprop.fullduplex;
+				if (strcmp(cur->intf.intf[i].name,
+				           pre->intf.intf[j].name) == 0)
+					break;
 			}
 
-			break;
+			// resync not succeeded?
+			if (! pre->intf.intf[j].name[0])
+			{
+				memcpy(&dev->intf.intf[i],
+				       &cur->intf.intf[i],
+				       sizeof cur->intf.intf[i]);
+
+				j = 0;
+				continue;
+			}
 		}
 
 		/*
@@ -825,47 +844,45 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev)
 		strcpy(dev->intf.intf[i].name, cur->intf.intf[i].name);
 
 		dev->intf.intf[i].rbyte = subcount(cur->intf.intf[i].rbyte,
-           	                                   pre->intf.intf[i].rbyte);
+           	                                   pre->intf.intf[j].rbyte);
 		dev->intf.intf[i].rpack = subcount(cur->intf.intf[i].rpack,
-		                                   pre->intf.intf[i].rpack);
+		                                   pre->intf.intf[j].rpack);
 		dev->intf.intf[i].rerrs = subcount(cur->intf.intf[i].rerrs,
-		                                   pre->intf.intf[i].rerrs);
+		                                   pre->intf.intf[j].rerrs);
 		dev->intf.intf[i].rdrop = subcount(cur->intf.intf[i].rdrop,
-		                                   pre->intf.intf[i].rdrop);
+		                                   pre->intf.intf[j].rdrop);
 		dev->intf.intf[i].rfifo = subcount(cur->intf.intf[i].rfifo,
-		                                   pre->intf.intf[i].rfifo);
+		                                   pre->intf.intf[j].rfifo);
 		dev->intf.intf[i].rframe= subcount(cur->intf.intf[i].rframe,
-		                                   pre->intf.intf[i].rframe);
+		                                   pre->intf.intf[j].rframe);
 		dev->intf.intf[i].rcompr= subcount(cur->intf.intf[i].rcompr,
-		                                   pre->intf.intf[i].rcompr);
+		                                   pre->intf.intf[j].rcompr);
 		dev->intf.intf[i].rmultic=subcount(cur->intf.intf[i].rmultic,
-		                                   pre->intf.intf[i].rmultic);
+		                                   pre->intf.intf[j].rmultic);
 
 		dev->intf.intf[i].sbyte = subcount(cur->intf.intf[i].sbyte,
-		                                   pre->intf.intf[i].sbyte);
+		                                   pre->intf.intf[j].sbyte);
 		dev->intf.intf[i].spack = subcount(cur->intf.intf[i].spack,
-		                                   pre->intf.intf[i].spack);
+		                                   pre->intf.intf[j].spack);
 		dev->intf.intf[i].serrs = subcount(cur->intf.intf[i].serrs,
-		                                   pre->intf.intf[i].serrs);
+		                                   pre->intf.intf[j].serrs);
 		dev->intf.intf[i].sdrop = subcount(cur->intf.intf[i].sdrop,
-		                                   pre->intf.intf[i].sdrop);
+		                                   pre->intf.intf[j].sdrop);
 		dev->intf.intf[i].sfifo = subcount(cur->intf.intf[i].sfifo,
-		                                   pre->intf.intf[i].sfifo);
+		                                   pre->intf.intf[j].sfifo);
 		dev->intf.intf[i].scollis= subcount(cur->intf.intf[i].scollis,
-		                                   pre->intf.intf[i].scollis);
+		                                   pre->intf.intf[j].scollis);
 		dev->intf.intf[i].scarrier= subcount(cur->intf.intf[i].scarrier,
-		                                   pre->intf.intf[i].scarrier);
+		                                   pre->intf.intf[j].scarrier);
 		dev->intf.intf[i].scompr= subcount(cur->intf.intf[i].scompr,
-		                                   pre->intf.intf[i].scompr);
+		                                   pre->intf.intf[j].scompr);
 
-		dev->intf.intf[i].speed 	= pre->intf.intf[i].speed;
-		dev->intf.intf[i].duplex	= pre->intf.intf[i].duplex;
+		dev->intf.intf[i].type  	= cur->intf.intf[i].type;
+		dev->intf.intf[i].duplex	= cur->intf.intf[i].duplex;
+		dev->intf.intf[i].speed 	= cur->intf.intf[i].speed;
+		dev->intf.intf[i].speedp 	= pre->intf.intf[j].speed;
 
-		/*
-		** save interface properties for next interval
-		*/
-		cur->intf.intf[i].speed 	= pre->intf.intf[i].speed;
-		cur->intf.intf[i].duplex	= pre->intf.intf[i].duplex;
+		cur->intf.intf[i].speedp 	= pre->intf.intf[j].speed;
 	}
 
 	dev->intf.intf[i].name[0] = '\0';
@@ -1039,6 +1056,170 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev)
 	dev->dsk.lvm[i].name[0] = '\0';
 	dev->dsk.nlvm = i;
 
+	/*
+	** calculate deviations for NFS
+	*/
+	dev->nfs.server.netcnt    = subcount(cur->nfs.server.netcnt,
+	                                     pre->nfs.server.netcnt);
+	dev->nfs.server.netudpcnt = subcount(cur->nfs.server.netudpcnt,
+	                                     pre->nfs.server.netudpcnt);
+	dev->nfs.server.nettcpcnt = subcount(cur->nfs.server.nettcpcnt,
+	                                     pre->nfs.server.nettcpcnt);
+	dev->nfs.server.nettcpcon = subcount(cur->nfs.server.nettcpcon,
+	                                     pre->nfs.server.nettcpcon);
+
+	dev->nfs.server.rpccnt    = subcount(cur->nfs.server.rpccnt,
+	                                     pre->nfs.server.rpccnt);
+	dev->nfs.server.rpcread   = subcount(cur->nfs.server.rpcread,
+	                                     pre->nfs.server.rpcread);
+	dev->nfs.server.rpcwrite  = subcount(cur->nfs.server.rpcwrite,
+	                                     pre->nfs.server.rpcwrite);
+	dev->nfs.server.rpcbadfmt = subcount(cur->nfs.server.rpcbadfmt,
+	                                     pre->nfs.server.rpcbadfmt);
+	dev->nfs.server.rpcbadaut = subcount(cur->nfs.server.rpcbadaut,
+	                                     pre->nfs.server.rpcbadaut);
+	dev->nfs.server.rpcbadcln = subcount(cur->nfs.server.rpcbadcln,
+	                                     pre->nfs.server.rpcbadcln);
+	
+	dev->nfs.server.rchits    = subcount(cur->nfs.server.rchits,
+	                                     pre->nfs.server.rchits);
+	dev->nfs.server.rcmiss    = subcount(cur->nfs.server.rcmiss,
+	                                     pre->nfs.server.rcmiss);
+	dev->nfs.server.rcnoca    = subcount(cur->nfs.server.rcnoca,
+	                                     pre->nfs.server.rcnoca);
+
+	dev->nfs.server.nrbytes   = subcount(cur->nfs.server.nrbytes,
+	                                     pre->nfs.server.nrbytes);
+	dev->nfs.server.nwbytes   = subcount(cur->nfs.server.nwbytes,
+	                                     pre->nfs.server.nwbytes);
+
+	dev->nfs.client.rpccnt        = subcount(cur->nfs.client.rpccnt,
+	                                         pre->nfs.client.rpccnt);
+	dev->nfs.client.rpcread       = subcount(cur->nfs.client.rpcread,
+	                                         pre->nfs.client.rpcread);
+	dev->nfs.client.rpcwrite      = subcount(cur->nfs.client.rpcwrite,
+	                                         pre->nfs.client.rpcwrite);
+	dev->nfs.client.rpcretrans    = subcount(cur->nfs.client.rpcretrans,
+	                                         pre->nfs.client.rpcretrans);
+	dev->nfs.client.rpcautrefresh = subcount(cur->nfs.client.rpcautrefresh,
+	                                         pre->nfs.client.rpcautrefresh);
+
+
+	for (i=j=0; i < cur->nfs.nfsmounts.nrmounts; i++, j++)
+	{
+		/*
+ 		** check if nfsmounts have been added or removed since
+		** previous interval
+		*/
+		if ( strcmp(cur->nfs.nfsmounts.nfsmnt[i].mountdev,
+		            pre->nfs.nfsmounts.nfsmnt[j].mountdev) != 0)
+		{
+			for (j=0; j < pre->nfs.nfsmounts.nrmounts; j++)
+			{
+			    if ( strcmp(cur->nfs.nfsmounts.nfsmnt[i].mountdev,
+		                        pre->nfs.nfsmounts.nfsmnt[j].mountdev)
+									== 0)
+					break;
+			}
+
+			/*
+			** either the corresponding entry has been found
+			** in the case that a container has been removed,
+			** or an empty entry has been found (all counters
+			** on zero) in the case that a container has
+			** been added during the last sample
+			*/
+		}
+
+		strcpy(dev->nfs.nfsmounts.nfsmnt[i].mountdev,
+		       cur->nfs.nfsmounts.nfsmnt[i].mountdev);
+
+                dev->nfs.nfsmounts.nfsmnt[i].age = 
+                                    cur->nfs.nfsmounts.nfsmnt[i].age;
+
+		if (dev->nfs.nfsmounts.nfsmnt[i].age <= interval)
+			memset(&(pre->nfs.nfsmounts.nfsmnt[j]), 0, 
+					sizeof(struct pernfsmount));
+
+                dev->nfs.nfsmounts.nfsmnt[i].bytesread = 
+                          subcount(cur->nfs.nfsmounts.nfsmnt[i].bytesread,
+                                   pre->nfs.nfsmounts.nfsmnt[j].bytesread);
+
+                dev->nfs.nfsmounts.nfsmnt[i].byteswrite = 
+                          subcount(cur->nfs.nfsmounts.nfsmnt[i].byteswrite,
+                                   pre->nfs.nfsmounts.nfsmnt[j].byteswrite);
+
+                dev->nfs.nfsmounts.nfsmnt[i].bytesdread = 
+                          subcount(cur->nfs.nfsmounts.nfsmnt[i].bytesdread,
+                                   pre->nfs.nfsmounts.nfsmnt[j].bytesdread);
+
+                dev->nfs.nfsmounts.nfsmnt[i].bytesdwrite = 
+                          subcount(cur->nfs.nfsmounts.nfsmnt[i].bytesdwrite,
+                                   pre->nfs.nfsmounts.nfsmnt[j].bytesdwrite);
+
+                dev->nfs.nfsmounts.nfsmnt[i].bytestotread = 
+                          subcount(cur->nfs.nfsmounts.nfsmnt[i].bytestotread,
+                                   pre->nfs.nfsmounts.nfsmnt[j].bytestotread);
+
+                dev->nfs.nfsmounts.nfsmnt[i].bytestotwrite = 
+                          subcount(cur->nfs.nfsmounts.nfsmnt[i].bytestotwrite,
+                                   pre->nfs.nfsmounts.nfsmnt[j].bytestotwrite);
+
+                dev->nfs.nfsmounts.nfsmnt[i].pagesmread = 
+                          subcount(cur->nfs.nfsmounts.nfsmnt[i].pagesmread,
+                                   pre->nfs.nfsmounts.nfsmnt[j].pagesmread);
+
+                dev->nfs.nfsmounts.nfsmnt[i].pagesmwrite = 
+                          subcount(cur->nfs.nfsmounts.nfsmnt[i].pagesmwrite,
+                                   pre->nfs.nfsmounts.nfsmnt[j].pagesmwrite);
+	}
+
+	dev->nfs.nfsmounts.nrmounts = cur->nfs.nfsmounts.nrmounts;
+
+	/*
+	** calculate deviations for containers
+	*/
+	for (i=j=0; i < cur->cfs.nrcontainer; i++, j++)
+	{
+		/*
+ 		** check if containers have been added or removed since
+		** previous interval
+		*/
+		if (cur->cfs.cont[i].ctid != pre->cfs.cont[j].ctid)
+		{
+			for (j=0; j < pre->cfs.nrcontainer; j++)
+			{
+				if (cur->cfs.cont[i].ctid ==
+						pre->cfs.cont[j].ctid)
+					break;
+			}
+
+			/*
+			** either the corresponding entry has been found
+			** in the case that a container has been removed,
+			** or an empty entry has been found (all counters
+			** on zero) in the case that a container has
+			** been added during the last sample
+			*/
+		}
+
+		dev->cfs.cont[i].ctid    = cur->cfs.cont[i].ctid;
+		dev->cfs.cont[i].numproc = cur->cfs.cont[i].numproc;
+
+		dev->cfs.cont[i].system  = subcount(cur->cfs.cont[i].system,
+		                                    pre->cfs.cont[i].system);
+		dev->cfs.cont[i].user    = subcount(cur->cfs.cont[i].user,
+		                                    pre->cfs.cont[i].user);
+		dev->cfs.cont[i].nice    = subcount(cur->cfs.cont[i].nice,
+		                                    pre->cfs.cont[i].nice);
+		dev->cfs.cont[i].uptime  = subcount(cur->cfs.cont[i].uptime,
+		                                    pre->cfs.cont[i].uptime);
+
+		dev->cfs.cont[i].physpages = cur->cfs.cont[i].physpages;
+	}
+
+	dev->cfs.nrcontainer = cur->cfs.nrcontainer;
+
 	/*
 	** application-specific counters
 	*/
@@ -1139,8 +1320,33 @@ totalsyst(char category, struct sstat *new, struct sstat *tot)
 		break;
 
 	   case 'n':	/* accumulate network-related counters */
+		tot->nfs.server.rpccnt     += new->nfs.server.rpccnt;
+		tot->nfs.server.rpcread    += new->nfs.server.rpcread;
+		tot->nfs.server.rpcwrite   += new->nfs.server.rpcwrite;
+		tot->nfs.server.rpcbadfmt  += new->nfs.server.rpcbadfmt;
+		tot->nfs.server.rpcbadaut  += new->nfs.server.rpcbadaut;
+		tot->nfs.server.rpcbadcln  += new->nfs.server.rpcbadcln;
+
+		tot->nfs.server.netcnt     += new->nfs.server.netcnt;
+		tot->nfs.server.nettcpcnt  += new->nfs.server.nettcpcnt;
+		tot->nfs.server.netudpcnt  += new->nfs.server.netudpcnt;
+		tot->nfs.server.nettcpcon  += new->nfs.server.nettcpcon;
+
+		tot->nfs.server.rchits     += new->nfs.server.rchits;
+		tot->nfs.server.rcmiss     += new->nfs.server.rcmiss;
+		tot->nfs.server.rcnoca     += new->nfs.server.rcnoca;
+
+		tot->nfs.server.nrbytes    += new->nfs.server.nrbytes;
+		tot->nfs.server.nwbytes    += new->nfs.server.nwbytes;
+
+		tot->nfs.client.rpccnt        += new->nfs.client.rpccnt;
+		tot->nfs.client.rpcread       += new->nfs.client.rpcread;
+		tot->nfs.client.rpcwrite      += new->nfs.client.rpcwrite;
+		tot->nfs.client.rpcretrans    += new->nfs.client.rpcretrans;
+		tot->nfs.client.rpcautrefresh += new->nfs.client.rpcautrefresh;
+
 		/*
-		** structures with network-related counters are considered
+		** other structures with network counters are considered
 		** as tables of frequency-counters that will be accumulated;
 		** values that do not represent a frequency are corrected
 		** afterwards
@@ -1259,6 +1465,7 @@ totalsyst(char category, struct sstat *new, struct sstat *tot)
 			tot->intf.intf[i].scarrier+= new->intf.intf[i].scarrier;
 			tot->intf.intf[i].scompr  += new->intf.intf[i].scompr;
 	
+			tot->intf.intf[i].type     = new->intf.intf[i].type;
 			tot->intf.intf[i].speed    = new->intf.intf[i].speed;
 			tot->intf.intf[i].duplex   = new->intf.intf[i].duplex;
 		}
