@@ -452,6 +452,7 @@ photosyst(struct sstat *si)
 
                                 if (f > maxfreq)
                         		maxfreq=f;
+                        	didone=1;
                         }
 
 	                si->cpu.cpu[i].freqcnt.maxfreq	= maxfreq;
@@ -459,7 +460,6 @@ photosyst(struct sstat *si)
 	                si->cpu.cpu[i].freqcnt.ticks	= hits;
 
                         fclose(fp);
-                        didone=1;
                 }
 		else
 		{    // governor statistics not available
@@ -475,7 +475,6 @@ photosyst(struct sstat *si)
 	                                si->cpu.cpu[i].freqcnt.maxfreq =f/1000; 
                                 }
 
-                                didone=1;
                                 fclose(fp);
                         }
                         else 
@@ -494,17 +493,15 @@ photosyst(struct sstat *si)
    					// convert KHz to MHz
                                         si->cpu.cpu[i].freqcnt.cnt   = f/1000;
                                         si->cpu.cpu[i].freqcnt.ticks = 0;
+                                        didone=1;
                                 }
 
                                 fclose(fp);
-                                didone=1;
                         }
                         else
                         {
-                                si->cpu.cpu[i].freqcnt.maxfreq	= 0;
                                 si->cpu.cpu[i].freqcnt.cnt	= 0;
                                 si->cpu.cpu[i].freqcnt.ticks 	= 0;
-                                break;    // use cpuinfo
                         }
                 }
         } // for all CPUs
@@ -612,10 +609,7 @@ photosyst(struct sstat *si)
 
 	if ( (fp = fopen("meminfo", "r")) != NULL)
 	{
-		int	nrfields = 12;	/* number of fields to be filled */
-
-		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL && 
-								nrfields > 0)
+		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
 		{
 			nr = sscanf(linebuf,
 				"%s %lld %lld %lld %lld %lld %lld %lld "
@@ -633,13 +627,11 @@ photosyst(struct sstat *si)
 				si->mem.physmem	 	= cnts[0] / pagesize; 
 				si->mem.freemem		= cnts[2] / pagesize;
 				si->mem.buffermem	= cnts[4] / pagesize;
-				nrfields -= 3;
 			}
 			else	if ( strcmp("Swap:", nam) == EQ)
 				{
 					si->mem.totswap  = cnts[0] / pagesize;
 					si->mem.freeswap = cnts[2] / pagesize;
-					nrfields -= 2;
 				}
 			else	if (strcmp("Cached:", nam) == EQ)
 				{
@@ -647,14 +639,12 @@ photosyst(struct sstat *si)
 					{
 						si->mem.cachemem  =
 							cnts[0]*1024/pagesize;
-						nrfields--;
 					}
 				}
 			else	if (strcmp("Dirty:", nam) == EQ)
 				{
 					si->mem.cachedrt  =
 							cnts[0]*1024/pagesize;
-					nrfields--;
 				}
 			else	if (strcmp("MemTotal:", nam) == EQ)
 				{
@@ -662,7 +652,6 @@ photosyst(struct sstat *si)
 					{
 						si->mem.physmem  =
 							cnts[0]*1024/pagesize;
-						nrfields--;
 					}
 				}
 			else	if (strcmp("MemFree:", nam) == EQ)
@@ -671,7 +660,6 @@ photosyst(struct sstat *si)
 					{
 						si->mem.freemem  =
 							cnts[0]*1024/pagesize;
-						nrfields--;
 					}
 				}
 			else	if (strcmp("Buffers:", nam) == EQ)
@@ -680,13 +668,11 @@ photosyst(struct sstat *si)
 					{
 						si->mem.buffermem  =
 							cnts[0]*1024/pagesize;
-						nrfields--;
 					}
 				}
 			else	if (strcmp("Shmem:", nam) == EQ)
 				{
 					si->mem.shmem = cnts[0]*1024/pagesize;
-					nrfields--;
 				}
 			else	if (strcmp("SwapTotal:", nam) == EQ)
 				{
@@ -694,7 +680,6 @@ photosyst(struct sstat *si)
 					{
 						si->mem.totswap  =
 							cnts[0]*1024/pagesize;
-						nrfields--;
 					}
 				}
 			else	if (strcmp("SwapFree:", nam) == EQ)
@@ -703,32 +688,61 @@ photosyst(struct sstat *si)
 					{
 						si->mem.freeswap  =
 							cnts[0]*1024/pagesize;
-						nrfields--;
 					}
 				}
 			else	if (strcmp("Slab:", nam) == EQ)
 				{
 					si->mem.slabmem = cnts[0]*1024/pagesize;
-					nrfields--;
 				}
 			else	if (strcmp("SReclaimable:", nam) == EQ)
 				{
 					si->mem.slabreclaim = cnts[0]*1024/
 								pagesize;
-					nrfields--;
 				}
 			else	if (strcmp("Committed_AS:", nam) == EQ)
 				{
 					si->mem.committed = cnts[0]*1024/
 								pagesize;
-					nrfields--;
 				}
 			else	if (strcmp("CommitLimit:", nam) == EQ)
 				{
 					si->mem.commitlim = cnts[0]*1024/
 								pagesize;
-					nrfields--;
 				}
+			else	if (strcmp("HugePages_Total:", nam) == EQ)
+				{
+					si->mem.tothugepage = cnts[0];
+				}
+			else	if (strcmp("HugePages_Free:", nam) == EQ)
+				{
+					si->mem.freehugepage = cnts[0];
+				}
+			else	if (strcmp("Hugepagesize:", nam) == EQ)
+				{
+					si->mem.hugepagesz = cnts[0]*1024;
+				}
+		}
+
+		fclose(fp);
+	}
+
+	/*
+	** gather vmware-related statistics from /proc/vmmemctl
+	** (only present if balloon driver enabled)
+	*/ 
+	si->mem.vmwballoon = (count_t) 0;
+
+	if ( (fp = fopen("vmmemctl", "r")) != NULL)
+	{
+		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+		{
+			nr = sscanf(linebuf, "%s %lld ", nam, &cnts[0]);
+
+			if ( strcmp("current:", nam) == EQ)
+			{
+				si->mem.vmwballoon = cnts[0];
+				break;
+			}
 		}
 
 		fclose(fp);
@@ -1018,6 +1032,338 @@ photosyst(struct sstat *si)
 		si->mem.shmswp = shminfo.shm_swp;
 	}
 
+	/*
+	** NFS server statistics
+	*/
+	if ( (fp = fopen("net/rpc/nfsd", "r")) != NULL)
+	{
+		char    label[32];
+		count_t	cnt[40];
+
+		/*
+		** every line starts with a small label,
+		** followed by upto 60 counters
+		*/
+		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+		{
+			memset(cnt, 0, sizeof cnt);
+
+			nr = sscanf(linebuf, "%31s %lld %lld %lld %lld %lld"
+			                          "%lld %lld %lld %lld %lld"
+			                          "%lld %lld %lld %lld %lld"
+			                          "%lld %lld %lld %lld %lld"
+			                          "%lld %lld %lld %lld %lld"
+			                          "%lld %lld %lld %lld %lld"
+			                          "%lld %lld %lld %lld %lld"
+			                          "%lld %lld %lld %lld %lld",
+			            label,
+			            &cnt[0],  &cnt[1],  &cnt[2],  &cnt[3],
+			            &cnt[4],  &cnt[5],  &cnt[6],  &cnt[7],
+			            &cnt[8],  &cnt[9],  &cnt[10], &cnt[11],
+			            &cnt[12], &cnt[13], &cnt[14], &cnt[15],
+			            &cnt[16], &cnt[17], &cnt[18], &cnt[19],
+			            &cnt[20], &cnt[21], &cnt[22], &cnt[23],
+			            &cnt[24], &cnt[25], &cnt[26], &cnt[27],
+			            &cnt[28], &cnt[29], &cnt[30], &cnt[31],
+			            &cnt[32], &cnt[33], &cnt[34], &cnt[35],
+			            &cnt[36], &cnt[37], &cnt[38], &cnt[39]);
+
+			if (nr < 2)		// unexpected empty line ?
+				continue;
+
+		   	if (strcmp(label, "rc") == 0)
+		   	{
+				si->nfs.server.rchits = cnt[0];
+				si->nfs.server.rcmiss = cnt[1];
+				si->nfs.server.rcnoca = cnt[2];
+
+				continue;
+			}
+
+		   	if (strcmp(label, "io") == 0)
+		   	{
+				si->nfs.server.nrbytes = cnt[0];
+				si->nfs.server.nwbytes = cnt[1];
+
+				continue;
+			}
+
+		   	if (strcmp(label, "net") == 0)
+		   	{
+				si->nfs.server.netcnt    = cnt[0];
+				si->nfs.server.netudpcnt = cnt[1];
+				si->nfs.server.nettcpcnt = cnt[2];
+				si->nfs.server.nettcpcon = cnt[3];
+
+				continue;
+			}
+
+		   	if (strcmp(label, "rpc") == 0)
+		   	{
+				si->nfs.server.rpccnt    = cnt[0];
+				si->nfs.server.rpcbadfmt = cnt[1];
+				si->nfs.server.rpcbadaut = cnt[2];
+				si->nfs.server.rpcbadcln = cnt[3];
+
+				continue;
+			}
+			//
+			// first counter behind 'proc..' is number of
+			// counters that follow
+		   	if (strcmp(label, "proc2") == 0)
+		   	{
+				si->nfs.server.rpcread  += cnt[7]; // offset+1
+				si->nfs.server.rpcwrite += cnt[9]; // offset+1
+				continue;
+			}
+		   	if (strcmp(label, "proc3") == 0)
+		   	{
+				si->nfs.server.rpcread  += cnt[7]; // offset+1
+				si->nfs.server.rpcwrite += cnt[8]; // offset+1
+				continue;
+			}
+		   	if (strcmp(label, "proc4ops") == 0)
+		   	{
+				si->nfs.server.rpcread  += cnt[26]; // offset+1
+				si->nfs.server.rpcwrite += cnt[39]; // offset+1
+				continue;
+			}
+		}
+
+		fclose(fp);
+	}
+
+	/*
+	** NFS client statistics
+	*/
+	if ( (fp = fopen("net/rpc/nfs", "r")) != NULL)
+	{
+		char    label[32];
+		count_t	cnt[10];
+
+		/*
+		** every line starts with a small label,
+		** followed by counters
+		*/
+		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+		{
+			memset(cnt, 0, sizeof cnt);
+
+			nr = sscanf(linebuf, "%31s %lld %lld %lld %lld %lld"
+			                          "%lld %lld %lld %lld %lld",
+			            label,
+			            &cnt[0], &cnt[1], &cnt[2], &cnt[3],
+			            &cnt[4], &cnt[5], &cnt[6], &cnt[7],
+			            &cnt[8], &cnt[9]);
+
+			if (nr < 2)		// unexpected empty line ?
+				continue;
+
+		   	if (strcmp(label, "rpc") == 0)
+		   	{
+				si->nfs.client.rpccnt        = cnt[0];
+				si->nfs.client.rpcretrans    = cnt[1];
+				si->nfs.client.rpcautrefresh = cnt[2];
+				continue;
+			}
+
+			// first counter behind 'proc..' is number of
+			// counters that follow
+		   	if (strcmp(label, "proc2") == 0)
+		   	{
+				si->nfs.client.rpcread  += cnt[7]; // offset+1
+				si->nfs.client.rpcwrite += cnt[9]; // offset+1
+				continue;
+			}
+		   	if (strcmp(label, "proc3") == 0)
+		   	{
+				si->nfs.client.rpcread  += cnt[7]; // offset+1
+				si->nfs.client.rpcwrite += cnt[8]; // offset+1
+				continue;
+			}
+		   	if (strcmp(label, "proc4") == 0)
+		   	{
+				si->nfs.client.rpcread  += cnt[2]; // offset+1
+				si->nfs.client.rpcwrite += cnt[3]; // offset+1
+				continue;
+			}
+		}
+
+		fclose(fp);
+	}
+
+	/*
+	** NFS client: per-mount statistics
+	*/
+	regainrootprivs();
+
+	if ( (fp = fopen("self/mountstats", "r")) != NULL)
+	{
+		char 	mountdev[128], fstype[32], label[32];
+                count_t	cnt[8];
+
+		i = 0;
+
+		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+		{
+			// if 'device' line, just remember the mounted device
+			if (sscanf(linebuf,
+				"device %127s mounted on %*s with fstype %31s",
+				mountdev, fstype) == 2)
+			{
+				continue;
+			}
+
+			if (memcmp(fstype, "nfs", 3) != 0)
+				continue;
+
+			// this is line with NFS client stats
+			nr = sscanf(linebuf,
+                                "%31s %lld %lld %lld %lld %lld %lld %lld %lld",
+				label, &cnt[0], &cnt[1], &cnt[2], &cnt[3],
+				       &cnt[4], &cnt[5], &cnt[6], &cnt[7]);
+
+			if (nr >= 2 )
+			{
+		   		if (strcmp(label, "age:") == 0)
+				{
+				    strcpy(si->nfs.nfsmounts.nfsmnt[i].mountdev,
+				           mountdev);
+
+				    si->nfs.nfsmounts.nfsmnt[i].age = cnt[0];
+				}
+
+		   		if (strcmp(label, "bytes:") == 0)
+				{
+				    si->nfs.nfsmounts.nfsmnt[i].bytesread =
+									cnt[0];
+				    si->nfs.nfsmounts.nfsmnt[i].byteswrite =
+									cnt[1];
+				    si->nfs.nfsmounts.nfsmnt[i].bytesdread =
+									cnt[2];
+				    si->nfs.nfsmounts.nfsmnt[i].bytesdwrite =
+									cnt[3];
+				    si->nfs.nfsmounts.nfsmnt[i].bytestotread =
+									cnt[4];
+				    si->nfs.nfsmounts.nfsmnt[i].bytestotwrite =
+									cnt[5];
+				    si->nfs.nfsmounts.nfsmnt[i].pagesmread =
+									cnt[6];
+				    si->nfs.nfsmounts.nfsmnt[i].pagesmwrite =
+									cnt[7];
+
+				    if (++i >= MAXNFSMOUNT-1)
+					break;
+				}
+			}
+		}
+
+		si->nfs.nfsmounts.nrmounts = i;
+
+		fclose(fp);
+	}
+
+	if (! droprootprivs())
+		cleanstop(42);
+
+	/*
+	** Container statistics (if any)
+	*/
+	if ( (fp = fopen("user_beancounters", "r")) != NULL)
+	{
+		unsigned long	ctid;
+		char    	label[32];
+		count_t		cnt;
+
+		i = -1;
+
+		/*
+		** lines introducing a new container have an extra
+		** field with the container id at the beginning.
+		*/
+		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+		{
+			nr = sscanf(linebuf, "%lu: %31s %lld",
+			            		&ctid, label, &cnt);
+
+			if (nr == 3)		// new container ?
+			{
+				if (++i >= MAXCONTAINER)
+					break;
+
+				si->cfs.cont[i].ctid = ctid;
+			}
+			else
+			{
+				nr = sscanf(linebuf, "%31s %lld", label, &cnt);
+
+				if (nr != 2)
+					continue;
+			}
+
+			if (i == -1)	// no container defined yet
+				continue;
+
+	   		if (strcmp(label, "numproc") == 0)
+	   		{
+				si->cfs.cont[i].numproc = cnt;
+				continue;
+			}
+
+	   		if (strcmp(label, "physpages") == 0)
+	   		{
+				si->cfs.cont[i].physpages = cnt;
+				continue;
+			}
+		}
+
+		fclose(fp);
+
+		si->cfs.nrcontainer = i+1;
+
+		if ( (fp = fopen("vz/vestat", "r")) != NULL)
+		{
+			unsigned long	ctid;
+			count_t		cnt[8];
+
+			/*
+			** relevant lines start with container id
+			*/
+			while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+			{
+				nr = sscanf(linebuf, "%lu  %lld %lld %lld %lld"
+			                             "%lld %lld %lld %lld %lld",
+			            	&ctid,
+					&cnt[0], &cnt[1], &cnt[2], &cnt[3],
+			            	&cnt[4], &cnt[5], &cnt[6], &cnt[7],
+					&cnt[8]);
+
+				if (nr < 9)	// irrelevant contents
+					continue;
+
+				// relevant stats: search for containerid
+				for (i=0; i < si->cfs.nrcontainer; i++)
+				{
+					if (si->cfs.cont[i].ctid == ctid)
+						break;
+				}
+
+				if (i >= si->cfs.nrcontainer)
+					continue;	// container not found
+
+				si->cfs.cont[i].user   = cnt[0];
+				si->cfs.cont[i].nice   = cnt[1];
+				si->cfs.cont[i].system = cnt[2];
+				si->cfs.cont[i].uptime = cnt[3];
+			}
+
+			fclose(fp);
+		}
+	}
+
+	/*
+ 	** return to original directory
+	*/
 	if ( chdir(origdir) == -1)
 		cleanstop(53);
 
@@ -1181,14 +1527,13 @@ static struct {
 	{ "^sd[a-z][a-z]*$",			{0},  nullmodname, DSKTYPE, },
 	{ "^dm-[0-9][0-9]*$",			{0},  lvmmapname,  LVMTYPE, },
 	{ "^md[0-9][0-9]*$",			{0},  nullmodname, MDDTYPE, },
-	{ "^vd[a-z][a-z]*$",			{0},  nullmodname, DSKTYPE, },
+	{ "^vd[a-z][a-z]*$",                    {0},  nullmodname, DSKTYPE, },
 	{ "^hd[a-z]$",				{0},  nullmodname, DSKTYPE, },
 	{ "^rd/c[0-9][0-9]*d[0-9][0-9]*$",	{0},  nullmodname, DSKTYPE, },
 	{ "^cciss/c[0-9][0-9]*d[0-9][0-9]*$",	{0},  nullmodname, DSKTYPE, },
 	{ "^fio[a-z][a-z]*$",			{0},  nullmodname, DSKTYPE, },
 	{ "/host.*/bus.*/target.*/lun.*/disc",	{0},  abbrevname1, DSKTYPE, },
-	{ "^xvd[a-z][a-z]*$",			{0},  nullmodname, DSKTYPE, },
-	{ "^vd[a-z][a-z]*$",                    {0},  nullmodname, DSKTYPE, },
+	{ "^xvd[a-z][a-z]*[1-9][0-9]*$",	{0},  nullmodname, DSKTYPE, },
 	{ "^dasd[a-z][a-z]*$",			{0},  nullmodname, DSKTYPE, },
 	{ "^mmcblk[0-9][0-9]*$",		{0},  nullmodname, DSKTYPE, },
 	{ "^emcpower[a-z][a-z]*$",		{0},  nullmodname, DSKTYPE, },
