@@ -174,6 +174,9 @@ extern sys_printdef syspdef_RECSLAB;
 extern sys_printdef syspdef_SHMEM;
 extern sys_printdef syspdef_SHMRSS;
 extern sys_printdef syspdef_SHMSWP;
+extern sys_printdef syspdef_VMWBAL;
+extern sys_printdef syspdef_HUPTOT;
+extern sys_printdef syspdef_HUPUSE;
 extern sys_printdef syspdef_SWPTOT;
 extern sys_printdef syspdef_SWPFREE;
 extern sys_printdef syspdef_SWPCOMMITTED;
@@ -183,6 +186,10 @@ extern sys_printdef syspdef_PAGSTEAL;
 extern sys_printdef syspdef_PAGSTALL;
 extern sys_printdef syspdef_PAGSWIN;
 extern sys_printdef syspdef_PAGSWOUT;
+extern sys_printdef syspdef_CONTNAME;
+extern sys_printdef syspdef_CONTNPROC;
+extern sys_printdef syspdef_CONTCPU;
+extern sys_printdef syspdef_CONTMEM;
 extern sys_printdef syspdef_DSKNAME;
 extern sys_printdef syspdef_DSKBUSY;
 extern sys_printdef syspdef_DSKNREAD;
@@ -215,6 +222,7 @@ extern sys_printdef syspdef_NETICMPOUT;
 extern sys_printdef syspdef_NETNAME;
 extern sys_printdef syspdef_NETPCKI;
 extern sys_printdef syspdef_NETPCKO;
+extern sys_printdef syspdef_NETSPEEDMAX;
 extern sys_printdef syspdef_NETSPEEDIN;
 extern sys_printdef syspdef_NETSPEEDOUT;
 extern sys_printdef syspdef_NETCOLLIS;
@@ -223,6 +231,34 @@ extern sys_printdef syspdef_NETRCVERR;
 extern sys_printdef syspdef_NETSNDERR;
 extern sys_printdef syspdef_NETRCVDROP;
 extern sys_printdef syspdef_NETSNDDROP;
+extern sys_printdef syspdef_NFMSERVER;
+extern sys_printdef syspdef_NFMPATH;
+extern sys_printdef syspdef_NFMTOTREAD;
+extern sys_printdef syspdef_NFMTOTWRITE;
+extern sys_printdef syspdef_NFMNREAD;
+extern sys_printdef syspdef_NFMNWRITE;
+extern sys_printdef syspdef_NFMDREAD;
+extern sys_printdef syspdef_NFMDWRITE;
+extern sys_printdef syspdef_NFMMREAD;
+extern sys_printdef syspdef_NFMMWRITE;
+extern sys_printdef syspdef_NFCRPCCNT;
+extern sys_printdef syspdef_NFCRPCREAD;
+extern sys_printdef syspdef_NFCRPCWRITE;
+extern sys_printdef syspdef_NFCRPCRET;
+extern sys_printdef syspdef_NFCRPCARF;
+extern sys_printdef syspdef_NFSRPCCNT;
+extern sys_printdef syspdef_NFSRPCREAD;
+extern sys_printdef syspdef_NFSRPCWRITE;
+extern sys_printdef syspdef_NFSBADFMT;
+extern sys_printdef syspdef_NFSBADAUT;
+extern sys_printdef syspdef_NFSBADCLN;
+extern sys_printdef syspdef_NFSNETTCP;
+extern sys_printdef syspdef_NFSNETUDP;
+extern sys_printdef syspdef_NFSNRBYTES;
+extern sys_printdef syspdef_NFSNWBYTES;
+extern sys_printdef syspdef_NFSRCHITS;
+extern sys_printdef syspdef_NFSRCMISS;
+extern sys_printdef syspdef_NFSRCNOCA;
 extern sys_printdef syspdef_BLANKBOX;
 
 
@@ -247,6 +283,7 @@ extern proc_printdef procprt_MAJFLT;
 extern proc_printdef procprt_VSTEXT;
 extern proc_printdef procprt_VSIZE;
 extern proc_printdef procprt_RSIZE;
+extern proc_printdef procprt_PSIZE;
 extern proc_printdef procprt_VSLIBS;
 extern proc_printdef procprt_VDATA;
 extern proc_printdef procprt_VSTACK;
@@ -260,6 +297,8 @@ extern proc_printdef procprt_RGID;
 extern proc_printdef procprt_EGID;
 extern proc_printdef procprt_SGID;
 extern proc_printdef procprt_FSGID;
+extern proc_printdef procprt_CTID;
+extern proc_printdef procprt_VPID;
 extern proc_printdef procprt_STDATE;
 extern proc_printdef procprt_STTIME;
 extern proc_printdef procprt_ENDATE;
