@@ -203,9 +203,9 @@ daysecs(time_t itime)
 
 
 /*
-** Function val2valstr() converts a value to an ascii-string of a fixed
-** number of positions; if the value does not fit, it will be formatted to
-** exponent-notation (as short as possible, so not via the standard printf-
+** Function val2valstr() converts a positive value to an ascii-string of a 
+** fixed number of positions; if the value does not fit, it will be formatted
+** to exponent-notation (as short as possible, so not via the standard printf-
 ** formatters %f or %e). The offered string should have a length of width+1.
 ** The value might even be printed as an average for the interval-time.
 */
@@ -216,13 +216,19 @@ val2valstr(count_t value, char *strvalue, int width, int avg, int nsecs)
 	int	exp     = 0;
 	char	*suffix = "";
 
-	if (avg)
+	if (avg && nsecs)
 	{
 		value  = (value + (nsecs/2)) / nsecs;     /* rounded value */
 		width  = width - 2;	/* subtract two positions for '/s' */
 		suffix = "/s";
 	}
 
+	if (value < 0)		// no negative value expected
+	{
+		sprintf(strvalue, "%*s%s", width, "?", suffix);
+		return strvalue;
+	}
+
 	maxval = pow(10.0, width) - 1;
 
 	if (value < maxval)
@@ -278,34 +284,24 @@ val2valstr(count_t value, char *strvalue, int width, int avg, int nsecs)
 int
 val2elapstr(int value, char *strvalue)
 {
-        char	*p=strvalue, doshow=0;
+        char	*p=strvalue;
 
-        if (value > DAYSECS) 
+        if (value >= DAYSECS) 
         {
                 p+=sprintf(p, "%dd", value/DAYSECS);
-                value %= DAYSECS;
-		doshow = 1;
         }
 
-        if (value > HOURSECS || doshow) 
+        if (value >= HOURSECS) 
         {
-                p+=sprintf(p, "%dh", value/HOURSECS);
-                value %= HOURSECS;
-		doshow = 1;
+                p+=sprintf(p, "%dh", (value%DAYSECS)/HOURSECS);
         }
 
-        if (value > MINSECS || doshow) 
+        if (value >= MINSECS) 
         {
-                p+=sprintf(p, "%dm", value/MINSECS);
-                value %= MINSECS;
-		doshow = 1;
+                p+=sprintf(p, "%dm", (value%HOURSECS)/MINSECS);
         }
 
-        if (value || doshow) 
-        {
-                p+=sprintf(p, "%ds", value);
-		doshow = 1;
-        }
+        p+=sprintf(p, "%ds", (value%MINSECS));
 
         return p-strvalue;
 }
@@ -405,11 +401,13 @@ val2Hzstr(count_t value, char *strvalue)
 #define	ONEMBYTE	1048576
 #define	ONEGBYTE	1073741824L
 #define	ONETBYTE	1099511627776LL
+#define	ONEPBYTE	1125899906842624LL
 
 #define	MAXBYTE		1024
 #define	MAXKBYTE	ONEKBYTE*99999L
 #define	MAXMBYTE	ONEMBYTE*999L
 #define	MAXGBYTE	ONEGBYTE*999LL
+#define	MAXTBYTE	ONETBYTE*999LL
 
 char *
 val2memstr(count_t value, char *strvalue, int pformat, int avgval, int nsecs)
@@ -433,10 +431,10 @@ val2memstr(count_t value, char *strvalue, int pformat, int avgval, int nsecs)
 	/*
 	** verify if printed value is required per second (average) or total
 	*/
-	if (avgval)
+	if (avgval && nsecs)
 	{
 		value     /= nsecs;
-		verifyval *= 100;
+		verifyval  = verifyval * 100 /nsecs;
 		basewidth -= 2;
 		suffix     = "/s";
 	}
@@ -453,10 +451,13 @@ val2memstr(count_t value, char *strvalue, int pformat, int avgval, int nsecs)
 			if (verifyval <= MAXMBYTE)	/* mbytes ? */
 				aformat = MBFORMAT;
 			else
-				if (verifyval <= MAXGBYTE)	/* mbytes ? */
+				if (verifyval <= MAXGBYTE)	/* gbytes ? */
 					aformat = GBFORMAT;
 				else
-					aformat = TBFORMAT;
+				 	if (verifyval <= MAXTBYTE)/* tbytes? */
+						aformat = TBFORMAT;/* tbytes! */
+					else
+						aformat = PBFORMAT;/* pbytes! */
 
 	/*
 	** check if this is also the preferred format
@@ -491,6 +492,11 @@ val2memstr(count_t value, char *strvalue, int pformat, int avgval, int nsecs)
 			basewidth-1, (double)((double)value/ONETBYTE), suffix);
 		break;
 
+	   case	PBFORMAT:
+		sprintf(strvalue, "%*.1lfP%s",
+			basewidth-1, (double)((double)value/ONEPBYTE), suffix);
+		break;
+
 	   default:
 		sprintf(strvalue, "!TILT!");
 	}
